# Copyright 2011 OpenStack Foundation
# Copyright 2011 Nebula, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from positional import positional

from keystoneclient import base
from keystoneclient import exceptions
from keystoneclient.i18n import _


class Project(base.Resource):
    """Represents an Identity project.

    Attributes:
        * id: a uuid that identifies the project
        * name: project name
        * description: project description
        * enabled: boolean to indicate if project is enabled
        * parent_id: a uuid representing this project's parent in hierarchy
        * parents: a list or a structured dict containing the parents of this
                   project in the hierarchy
        * subtree: a list or a structured dict containing the subtree of this
                   project in the hierarchy

    """
    @positional(enforcement=positional.WARN)
    def update(self, name=None, description=None, enabled=None):
        kwargs = {
            'name': name if name is not None else self.name,
            'description': (description
                            if description is not None
                            else self.description),
            'enabled': enabled if enabled is not None else self.enabled,
        }

        try:
            retval = self.manager.update(self.id, **kwargs)
            self = retval
        except Exception:
            retval = None

        return retval


class ProjectManager(base.CrudManager):
    """Manager class for manipulating Identity projects."""
    resource_class = Project
    collection_key = 'projects'
    key = 'project'

    @positional(3, enforcement=positional.WARN)
    def create(self, name, domain, description=None,
               enabled=True, parent=None, **kwargs):
        """Create a project.

        :param str name: project name.
        :param domain: the project domain.
        :type domain: :py:class:`keystoneclient.v3.domains.Domain` or str
        :param str description: the project description. (optional)
        :param boolean enabled: if the project is enabled. (optional)
        :param parent: the project's parent in the hierarchy. (optional)
        :type parent: :py:class:`keystoneclient.v3.projects.Project` or str
        """

        # NOTE(rodrigods): the API must be backwards compatible, so if an
        # application was passing a 'parent_id' before as kwargs, the call
        # should not fail. If both 'parent' and 'parent_id' are provided,
        # 'parent' will be preferred.
        if parent:
            kwargs['parent_id'] = base.getid(parent)

        return super(ProjectManager, self).create(
            domain_id=base.getid(domain),
            name=name,
            description=description,
            enabled=enabled,
            **kwargs)

    @positional(enforcement=positional.WARN)
    def list(self, domain=None, user=None, **kwargs):
        """List projects.

        If domain or user are provided, then filter projects with
        those attributes.

        If ``**kwargs`` are provided, then filter projects with
        attributes matching ``**kwargs``.
        """
        base_url = '/users/%s' % base.getid(user) if user else None
        return super(ProjectManager, self).list(
            base_url=base_url,
            domain_id=base.getid(domain),
            fallback_to_auth=True,
            **kwargs)

    def _check_not_parents_as_ids_and_parents_as_list(self, parents_as_ids,
                                                      parents_as_list):
        if parents_as_ids and parents_as_list:
            msg = _('Specify either parents_as_ids or parents_as_list '
                    'parameters, not both')
            raise exceptions.ValidationError(msg)

    def _check_not_subtree_as_ids_and_subtree_as_list(self, subtree_as_ids,
                                                      subtree_as_list):
        if subtree_as_ids and subtree_as_list:
            msg = _('Specify either subtree_as_ids or subtree_as_list '
                    'parameters, not both')
            raise exceptions.ValidationError(msg)

    @positional()
    def get(self, project, subtree_as_list=False, parents_as_list=False,
            subtree_as_ids=False, parents_as_ids=False):
        """Get a project.

        :param project: project to be retrieved.
        :type project: :py:class:`keystoneclient.v3.projects.Project` or str
        :param boolean subtree_as_list: retrieve projects below this project
                                        in the hierarchy as a flat list.
                                        (optional)
        :param boolean parents_as_list: retrieve projects above this project
                                        in the hierarchy as a flat list.
                                        (optional)
        :param boolean subtree_as_ids: retrieve the IDs from the projects below
                                       this project in the hierarchy as a
                                       structured dictionary. (optional)
        :param boolean parents_as_ids: retrieve the IDs from the projects above
                                       this project in the hierarchy as a
                                       structured dictionary. (optional)

        :raises keystoneclient.exceptions.ValidationError: if subtree_as_list
            and subtree_as_ids or parents_as_list and parents_as_ids are
            included at the same time in the call.
        """
        self._check_not_parents_as_ids_and_parents_as_list(
            parents_as_ids, parents_as_list)
        self._check_not_subtree_as_ids_and_subtree_as_list(
            subtree_as_ids, subtree_as_list)

        # According to the API spec, the query params are key only
        query_params = []
        if subtree_as_list:
            query_params.append('subtree_as_list')
        if subtree_as_ids:
            query_params.append('subtree_as_ids')
        if parents_as_list:
            query_params.append('parents_as_list')
        if parents_as_ids:
            query_params.append('parents_as_ids')

        query = self.build_key_only_query(query_params)
        dict_args = {'project_id': base.getid(project)}
        url = self.build_url(dict_args_in_out=dict_args)
        return self._get(url + query, self.key)

    @positional(enforcement=positional.WARN)
    def update(self, project, name=None, domain=None, description=None,
               enabled=None, **kwargs):
        return super(ProjectManager, self).update(
            project_id=base.getid(project),
            domain_id=base.getid(domain),
            name=name,
            description=description,
            enabled=enabled,
            **kwargs)

    def delete(self, project):
        return super(ProjectManager, self).delete(
            project_id=base.getid(project))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from keystoneclient import client
import os_client_config


class ClientTestCase(testtools.TestCase):

    def setUp(self):
        super(ClientTestCase, self).setUp()

        self.client = self.get_client()

    def get_client(self):
        """Creates a keystoneclient instance to run functional tests

        The client is instantiated via os-client-config either based on a
        clouds.yaml config file or from the environment variables.

        First, look for a 'functional_admin' cloud, as this is a cloud that the
        user may have defined for functional testing with admin credentials. If
        that is not found, check for the 'devstack-admin' cloud. Finally, fall
        back to looking for environment variables.

        """
        IDENTITY_CLIENT = 'identity'
        OPENSTACK_CLOUDS = ('functional_admin', 'devstack-admin', 'envvars')

        for cloud in OPENSTACK_CLOUDS:
            try:
                return os_client_config.make_client(
                    IDENTITY_CLIENT, client.Client, cloud=cloud,
                    identity_api_version=self.version)
            except os_client_config.exceptions.OpenStackConfigException:
                pass

        raise Exception("Could not find any cloud definition for clouds named"
                        " functional_admin or devstack-admin. Check your"
                        " clouds.yaml file or your envvars and try again.")


class V3ClientTestCase(ClientTestCase):
    version = '3'


class V2ClientTestCase(ClientTestCase):
    version = '2.0'



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
from six import moves

from keystoneclient.generic import shell
from keystoneclient.tests.unit import utils


class DoDiscoverTest(utils.TestCase):
    """Unit tests for do_discover function."""
    foo_version = {
        'id': 'foo_id',
        'status': 'foo_status',
        'url': 'http://foo/url',
    }
    bar_version = {
        'id': 'bar_id',
        'status': 'bar_status',
        'url': 'http://bar/url',
    }
    foo_extension = {
        'foo': 'foo_extension',
        'message': 'extension_message',
        'bar': 'bar_extension',
    }
    stub_message = 'This is a stub message'

    def setUp(self):
        super(DoDiscoverTest, self).setUp()

        self.client_mock = mock.Mock()
        self.client_mock.discover.return_value = {}

    def _execute_discover(self):
        """Call do_discover function and capture output

        :returns: captured output is returned
        """
        with mock.patch('sys.stdout',
                        new_callable=moves.StringIO) as mock_stdout:
            shell.do_discover(self.client_mock, args=None)
            output = mock_stdout.getvalue()
        return output

    def _check_version_print(self, output, version):
        """Checks all api version's parameters are present in output."""
        self.assertIn(version['id'], output)
        self.assertIn(version['status'], output)
        self.assertIn(version['url'], output)

    def test_no_keystones(self):
        # No servers configured for client,
        # corresponding message should be printed
        output = self._execute_discover()
        self.assertIn('No Keystone-compatible endpoint found', output)

    def test_endpoint(self):
        # Endpoint is configured for client,
        # client's discover method should be called with that value
        self.client_mock.endpoint = 'Some non-empty value'
        shell.do_discover(self.client_mock, args=None)
        self.client_mock.discover.assert_called_with(self.client_mock.endpoint)

    def test_auth_url(self):
        # No endpoint provided for client, but there is an auth_url
        # client's discover method should be called with auth_url value
        self.client_mock.endpoint = False
        self.client_mock.auth_url = 'Some non-empty value'
        shell.do_discover(self.client_mock, args=None)
        self.client_mock.discover.assert_called_with(self.client_mock.auth_url)

    def test_empty(self):
        # No endpoint or auth_url is configured for client.
        # client.discover() should be called without parameters
        self.client_mock.endpoint = False
        self.client_mock.auth_url = False
        shell.do_discover(self.client_mock, args=None)
        self.client_mock.discover.assert_called_with()

    def test_message(self):
        # If client.discover() result contains message - it should be printed
        self.client_mock.discover.return_value = {'message': self.stub_message}
        output = self._execute_discover()
        self.assertIn(self.stub_message, output)

    def test_versions(self):
        # Every version in client.discover() result should be printed
        # and client.discover_extension() should be called on its url
        self.client_mock.discover.return_value = {
            'foo': self.foo_version,
            'bar': self.bar_version,
        }
        self.client_mock.discover_extensions.return_value = {}
        output = self._execute_discover()
        self._check_version_print(output, self.foo_version)
        self._check_version_print(output, self.bar_version)

        discover_extension_calls = [
            mock.call(self.foo_version['url']),
            mock.call(self.bar_version['url']),
        ]

        self.client_mock.discover_extensions.assert_has_calls(
            discover_extension_calls,
            any_order=True)

    def test_extensions(self):
        # Every extension's parameters should be printed
        # Extension's message should be omitted
        self.client_mock.discover.return_value = {'foo': self.foo_version}
        self.client_mock.discover_extensions.return_value = self.foo_extension
        output = self._execute_discover()
        self.assertIn(self.foo_extension['foo'], output)
        self.assertIn(self.foo_extension['bar'], output)
        self.assertNotIn(self.foo_extension['message'], output)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import argparse
import json
import logging
import os
import sys
import uuid

import fixtures
import mock
import six
import testtools
from testtools import matchers

from keystoneclient import exceptions
from keystoneclient import session
from keystoneclient import shell as openstack_shell
from keystoneclient.tests.unit import utils
from keystoneclient.v2_0 import shell as shell_v2_0


DEFAULT_USERNAME = 'username'
DEFAULT_PASSWORD = 'password'
DEFAULT_TENANT_ID = 'tenant_id'
DEFAULT_TENANT_NAME = 'tenant_name'
DEFAULT_AUTH_URL = 'http://127.0.0.1:5000/v2.0/'


# Make a fake shell object, a helping wrapper to call it
def shell(cmd):
    openstack_shell.OpenStackIdentityShell().main(cmd.split())


class NoExitArgumentParser(argparse.ArgumentParser):
    def error(self, message):
        raise exceptions.CommandError(message)


class ShellTest(utils.TestCase):

    FAKE_ENV = {
        'OS_USERNAME': DEFAULT_USERNAME,
        'OS_PASSWORD': DEFAULT_PASSWORD,
        'OS_TENANT_ID': DEFAULT_TENANT_ID,
        'OS_TENANT_NAME': DEFAULT_TENANT_NAME,
        'OS_AUTH_URL': DEFAULT_AUTH_URL,
    }

    def _tolerant_shell(self, cmd):
        t_shell = openstack_shell.OpenStackIdentityShell(NoExitArgumentParser)
        t_shell.main(cmd.split())

    # Patch os.environ to avoid required auth info.
    def setUp(self):

        super(ShellTest, self).setUp()
        for var in os.environ:
            if var.startswith("OS_"):
                self.useFixture(fixtures.EnvironmentVariable(var, ""))

        for var in self.FAKE_ENV:
            self.useFixture(fixtures.EnvironmentVariable(var,
                            self.FAKE_ENV[var]))

    def test_help_unknown_command(self):
        self.assertRaises(exceptions.CommandError, shell, 'help %s'
                          % uuid.uuid4().hex)

    def shell(self, argstr):
        orig = sys.stdout
        clean_env = {}
        _old_env, os.environ = os.environ, clean_env.copy()
        try:
            sys.stdout = six.StringIO()
            _shell = openstack_shell.OpenStackIdentityShell()
            _shell.main(argstr.split())
        except SystemExit:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            self.assertEqual(exc_value.code, 0)
        finally:
            out = sys.stdout.getvalue()
            sys.stdout.close()
            sys.stdout = orig
            os.environ = _old_env
        return out

    def test_help_no_args(self):
        do_tenant_mock = mock.MagicMock()
        with mock.patch('keystoneclient.shell.OpenStackIdentityShell.do_help',
                        do_tenant_mock):
            self.shell('')
            assert do_tenant_mock.called

    def test_help(self):
        required = 'usage:'
        help_text = self.shell('help')
        self.assertThat(help_text,
                        matchers.MatchesRegex(required))

    def test_help_command(self):
        required = 'usage: keystone user-create'
        help_text = self.shell('help user-create')
        self.assertThat(help_text,
                        matchers.MatchesRegex(required))

    def test_help_command_with_no_action_choices(self):
        required = 'usage: keystone user-update'
        help_text = self.shell('help user-update')
        self.assertThat(help_text,
                        matchers.MatchesRegex(required))

    def test_auth_no_credentials(self):
        with testtools.ExpectedException(
                exceptions.CommandError, 'Expecting'):
            self.shell('user-list')

    def test_debug(self):
        logging_mock = mock.MagicMock()
        with mock.patch('logging.basicConfig', logging_mock):
            self.assertRaises(exceptions.CommandError,
                              self.shell, '--debug user-list')
            self.assertTrue(logging_mock.called)
            self.assertEqual([(), {'level': logging.DEBUG}],
                             list(logging_mock.call_args))

    def test_auth_password_authurl_no_username(self):
        with testtools.ExpectedException(
                exceptions.CommandError,
                'Expecting a username provided via either'):
            self.shell('--os-password=%s --os-auth-url=%s user-list'
                       % (uuid.uuid4().hex, uuid.uuid4().hex))

    def test_auth_username_password_no_authurl(self):
        with testtools.ExpectedException(
                exceptions.CommandError, 'Expecting an auth URL via either'):
            self.shell('--os-password=%s --os-username=%s user-list'
                       % (uuid.uuid4().hex, uuid.uuid4().hex))

    def test_token_no_endpoint(self):
        with testtools.ExpectedException(
                exceptions.CommandError, 'Expecting an endpoint provided'):
            self.shell('--os-token=%s user-list' % uuid.uuid4().hex)

    def test_endpoint_no_token(self):
        with testtools.ExpectedException(
                exceptions.CommandError, 'Expecting a token provided'):
            self.shell('--os-endpoint=http://10.0.0.1:5000/v2.0/ user-list')

    def test_shell_args(self):
        do_tenant_mock = mock.MagicMock()
        with mock.patch('keystoneclient.v2_0.shell.do_user_list',
                        do_tenant_mock):
            shell('user-list')
            assert do_tenant_mock.called
            ((a, b), c) = do_tenant_mock.call_args
            actual = (b.os_auth_url, b.os_password, b.os_tenant_id,
                      b.os_tenant_name, b.os_username,
                      b.os_identity_api_version)
            expect = (DEFAULT_AUTH_URL, DEFAULT_PASSWORD, DEFAULT_TENANT_ID,
                      DEFAULT_TENANT_NAME, DEFAULT_USERNAME, '')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))

            # Old_style options
            shell('--os_auth_url http://0.0.0.0:5000/ --os_password xyzpdq '
                  '--os_tenant_id 1234 --os_tenant_name fred '
                  '--os_username barney '
                  '--os_identity_api_version 2.0 user-list')
            assert do_tenant_mock.called
            ((a, b), c) = do_tenant_mock.call_args
            actual = (b.os_auth_url, b.os_password, b.os_tenant_id,
                      b.os_tenant_name, b.os_username,
                      b.os_identity_api_version)
            expect = ('http://0.0.0.0:5000/', 'xyzpdq', '1234',
                      'fred', 'barney', '2.0')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))

            # New-style options
            shell('--os-auth-url http://1.1.1.1:5000/ --os-password xyzpdq '
                  '--os-tenant-id 4321 --os-tenant-name wilma '
                  '--os-username betty '
                  '--os-identity-api-version 2.0 user-list')
            assert do_tenant_mock.called
            ((a, b), c) = do_tenant_mock.call_args
            actual = (b.os_auth_url, b.os_password, b.os_tenant_id,
                      b.os_tenant_name, b.os_username,
                      b.os_identity_api_version)
            expect = ('http://1.1.1.1:5000/', 'xyzpdq', '4321',
                      'wilma', 'betty', '2.0')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))

            # Test keyring options
            shell('--os-auth-url http://1.1.1.1:5000/ --os-password xyzpdq '
                  '--os-tenant-id 4321 --os-tenant-name wilma '
                  '--os-username betty '
                  '--os-identity-api-version 2.0 '
                  '--os-cache '
                  '--stale-duration 500 '
                  '--force-new-token user-list')
            assert do_tenant_mock.called
            ((a, b), c) = do_tenant_mock.call_args
            actual = (b.os_auth_url, b.os_password, b.os_tenant_id,
                      b.os_tenant_name, b.os_username,
                      b.os_identity_api_version, b.os_cache,
                      b.stale_duration, b.force_new_token)
            expect = ('http://1.1.1.1:5000/', 'xyzpdq', '4321',
                      'wilma', 'betty', '2.0', True, '500', True)
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))

            # Test os-identity-api-version fall back to 2.0
            shell('--os-identity-api-version 3.0 user-list')
            assert do_tenant_mock.called
            self.assertTrue(b.os_identity_api_version, '2.0')

    def test_shell_user_create_args(self):
        """Test user-create args."""
        do_uc_mock = mock.MagicMock()
        # grab the decorators for do_user_create
        uc_func = getattr(shell_v2_0, 'do_user_create')
        do_uc_mock.arguments = getattr(uc_func, 'arguments', [])
        with mock.patch('keystoneclient.v2_0.shell.do_user_create',
                        do_uc_mock):

            # Old_style options
            # Test case with one --tenant_id args present: ec2 creds
            shell('user-create --name=FOO '
                  '--pass=secret --tenant_id=barrr --enabled=true')
            assert do_uc_mock.called
            ((a, b), c) = do_uc_mock.call_args
            actual = (b.os_auth_url, b.os_password, b.os_tenant_id,
                      b.os_tenant_name, b.os_username,
                      b.os_identity_api_version)
            expect = (DEFAULT_AUTH_URL, DEFAULT_PASSWORD, DEFAULT_TENANT_ID,
                      DEFAULT_TENANT_NAME, DEFAULT_USERNAME, '')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))
            actual = (b.tenant_id, b.name, b.passwd, b.enabled)
            expect = ('barrr', 'FOO', 'secret', 'true')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))

            # New-style options
            # Test case with one --tenant args present: ec2 creds
            shell('user-create --name=foo '
                  '--pass=secret --tenant=BARRR --enabled=true')
            assert do_uc_mock.called
            ((a, b), c) = do_uc_mock.call_args
            actual = (b.os_auth_url, b.os_password, b.os_tenant_id,
                      b.os_tenant_name, b.os_username,
                      b.os_identity_api_version)
            expect = (DEFAULT_AUTH_URL, DEFAULT_PASSWORD, DEFAULT_TENANT_ID,
                      DEFAULT_TENANT_NAME, DEFAULT_USERNAME, '')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))
            actual = (b.tenant, b.name, b.passwd, b.enabled)
            expect = ('BARRR', 'foo', 'secret', 'true')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))

            # New-style options
            # Test case with one --tenant-id args present: ec2 creds
            shell('user-create --name=foo '
                  '--pass=secret --tenant-id=BARRR --enabled=true')
            assert do_uc_mock.called
            ((a, b), c) = do_uc_mock.call_args
            actual = (b.os_auth_url, b.os_password, b.os_tenant_id,
                      b.os_tenant_name, b.os_username,
                      b.os_identity_api_version)
            expect = (DEFAULT_AUTH_URL, DEFAULT_PASSWORD, DEFAULT_TENANT_ID,
                      DEFAULT_TENANT_NAME, DEFAULT_USERNAME, '')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))
            actual = (b.tenant, b.name, b.passwd, b.enabled)
            expect = ('BARRR', 'foo', 'secret', 'true')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))

            # Old_style options
            # Test case with --os_tenant_id and --tenant_id args present
            shell('--os_tenant_id=os-tenant user-create --name=FOO '
                  '--pass=secret --tenant_id=barrr --enabled=true')
            assert do_uc_mock.called
            ((a, b), c) = do_uc_mock.call_args
            actual = (b.os_auth_url, b.os_password, b.os_tenant_id,
                      b.os_tenant_name, b.os_username,
                      b.os_identity_api_version)
            expect = (DEFAULT_AUTH_URL, DEFAULT_PASSWORD, 'os-tenant',
                      DEFAULT_TENANT_NAME, DEFAULT_USERNAME, '')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))
            actual = (b.tenant_id, b.name, b.passwd, b.enabled)
            expect = ('barrr', 'FOO', 'secret', 'true')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))

            # New-style options
            # Test case with --os-tenant-id and --tenant-id args present
            shell('--os-tenant-id=ostenant user-create --name=foo '
                  '--pass=secret --tenant-id=BARRR --enabled=true')
            assert do_uc_mock.called
            ((a, b), c) = do_uc_mock.call_args
            actual = (b.os_auth_url, b.os_password, b.os_tenant_id,
                      b.os_tenant_name, b.os_username,
                      b.os_identity_api_version)
            expect = (DEFAULT_AUTH_URL, DEFAULT_PASSWORD, 'ostenant',
                      DEFAULT_TENANT_NAME, DEFAULT_USERNAME, '')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))
            actual = (b.tenant, b.name, b.passwd, b.enabled)
            expect = ('BARRR', 'foo', 'secret', 'true')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))

    def test_do_tenant_create(self):
        do_tenant_mock = mock.MagicMock()
        with mock.patch('keystoneclient.v2_0.shell.do_tenant_create',
                        do_tenant_mock):
            shell('tenant-create')
            assert do_tenant_mock.called
            # FIXME(dtroyer): how do you test the decorators?
            # shell('tenant-create --tenant-name wilma '
            #        '--description "fred\'s wife"')
            # assert do_tenant_mock.called

    def test_do_tenant_list(self):
        do_tenant_mock = mock.MagicMock()
        with mock.patch('keystoneclient.v2_0.shell.do_tenant_list',
                        do_tenant_mock):
            shell('tenant-list')
            assert do_tenant_mock.called

    def test_shell_tenant_id_args(self):
        """Test where tenant_id is passed twice.

        Test a corner case where --tenant_id appears on the
        command-line twice.
        """
        do_ec2_mock = mock.MagicMock()
        # grab the decorators for do_ec2_create_credentials
        ec2_func = getattr(shell_v2_0, 'do_ec2_credentials_create')
        do_ec2_mock.arguments = getattr(ec2_func, 'arguments', [])
        with mock.patch('keystoneclient.v2_0.shell.do_ec2_credentials_create',
                        do_ec2_mock):

            # Old_style options
            # Test case with one --tenant_id args present: ec2 creds
            shell('ec2-credentials-create '
                  '--tenant_id=ec2-tenant --user_id=ec2-user')
            assert do_ec2_mock.called
            ((a, b), c) = do_ec2_mock.call_args
            actual = (b.os_auth_url, b.os_password, b.os_tenant_id,
                      b.os_tenant_name, b.os_username,
                      b.os_identity_api_version)
            expect = (DEFAULT_AUTH_URL, DEFAULT_PASSWORD, DEFAULT_TENANT_ID,
                      DEFAULT_TENANT_NAME, DEFAULT_USERNAME, '')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))
            actual = (b.tenant_id, b.user_id)
            expect = ('ec2-tenant', 'ec2-user')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))

            # New-style options
            # Test case with one --tenant-id args present: ec2 creds
            shell('ec2-credentials-create '
                  '--tenant-id=dash-tenant --user-id=dash-user')
            assert do_ec2_mock.called
            ((a, b), c) = do_ec2_mock.call_args
            actual = (b.os_auth_url, b.os_password, b.os_tenant_id,
                      b.os_tenant_name, b.os_username,
                      b.os_identity_api_version)
            expect = (DEFAULT_AUTH_URL, DEFAULT_PASSWORD, DEFAULT_TENANT_ID,
                      DEFAULT_TENANT_NAME, DEFAULT_USERNAME, '')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))
            actual = (b.tenant_id, b.user_id)
            expect = ('dash-tenant', 'dash-user')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))

            # Old_style options
            # Test case with two --tenant_id args present
            shell('--os_tenant_id=os-tenant ec2-credentials-create '
                  '--tenant_id=ec2-tenant --user_id=ec2-user')
            assert do_ec2_mock.called
            ((a, b), c) = do_ec2_mock.call_args
            actual = (b.os_auth_url, b.os_password, b.os_tenant_id,
                      b.os_tenant_name, b.os_username,
                      b.os_identity_api_version)
            expect = (DEFAULT_AUTH_URL, DEFAULT_PASSWORD, 'os-tenant',
                      DEFAULT_TENANT_NAME, DEFAULT_USERNAME, '')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))
            actual = (b.tenant_id, b.user_id)
            expect = ('ec2-tenant', 'ec2-user')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))

            # New-style options
            # Test case with two --tenant-id args present
            shell('--os-tenant-id=ostenant ec2-credentials-create '
                  '--tenant-id=dash-tenant --user-id=dash-user')
            assert do_ec2_mock.called
            ((a, b), c) = do_ec2_mock.call_args
            actual = (b.os_auth_url, b.os_password, b.os_tenant_id,
                      b.os_tenant_name, b.os_username,
                      b.os_identity_api_version)
            expect = (DEFAULT_AUTH_URL, DEFAULT_PASSWORD, 'ostenant',
                      DEFAULT_TENANT_NAME, DEFAULT_USERNAME, '')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))
            actual = (b.tenant_id, b.user_id)
            expect = ('dash-tenant', 'dash-user')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))

    def test_do_ec2_get(self):
        do_shell_mock = mock.MagicMock()

        with mock.patch('keystoneclient.v2_0.shell.do_ec2_credentials_create',
                        do_shell_mock):
            shell('ec2-credentials-create')
            assert do_shell_mock.called

        with mock.patch('keystoneclient.v2_0.shell.do_ec2_credentials_get',
                        do_shell_mock):
            shell('ec2-credentials-get')
            assert do_shell_mock.called

        with mock.patch('keystoneclient.v2_0.shell.do_ec2_credentials_list',
                        do_shell_mock):
            shell('ec2-credentials-list')
            assert do_shell_mock.called

        with mock.patch('keystoneclient.v2_0.shell.do_ec2_credentials_delete',
                        do_shell_mock):
            shell('ec2-credentials-delete')
            assert do_shell_mock.called

    def test_timeout_parse_invalid_type(self):
        for f in ['foobar', 'xyz']:
            cmd = '--timeout %s endpoint-create' % (f)
            self.assertRaises(exceptions.CommandError,
                              self._tolerant_shell, cmd)

    def test_timeout_parse_invalid_number(self):
        for f in [-1, 0]:
            cmd = '--timeout %s endpoint-create' % (f)
            self.assertRaises(exceptions.CommandError,
                              self._tolerant_shell, cmd)

    def test_do_timeout(self):
        response_mock = mock.MagicMock()
        response_mock.status_code = 200
        response_mock.text = json.dumps({
            'endpoints': [],
        })
        request_mock = mock.MagicMock(return_value=response_mock)
        with mock.patch.object(session.requests, 'request',
                               request_mock):
            shell(('--timeout 2 --os-token=blah  --os-endpoint=blah'
                   ' --os-auth-url=blah.com endpoint-list'))
            request_mock.assert_called_with(mock.ANY, mock.ANY,
                                            timeout=2,
                                            allow_redirects=False,
                                            headers=mock.ANY,
                                            verify=mock.ANY)

    def test_do_endpoints(self):
        do_shell_mock = mock.MagicMock()
        # grab the decorators for do_endpoint_create
        shell_func = getattr(shell_v2_0, 'do_endpoint_create')
        do_shell_mock.arguments = getattr(shell_func, 'arguments', [])
        with mock.patch('keystoneclient.v2_0.shell.do_endpoint_create',
                        do_shell_mock):

            # Old_style options
            # Test create args
            shell('endpoint-create '
                  '--service_id=2 --publicurl=http://example.com:1234/go '
                  '--adminurl=http://example.com:9876/adm')
            assert do_shell_mock.called
            ((a, b), c) = do_shell_mock.call_args
            actual = (b.os_auth_url, b.os_password, b.os_tenant_id,
                      b.os_tenant_name, b.os_username,
                      b.os_identity_api_version)
            expect = (DEFAULT_AUTH_URL, DEFAULT_PASSWORD, DEFAULT_TENANT_ID,
                      DEFAULT_TENANT_NAME, DEFAULT_USERNAME, '')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))
            actual = (b.service, b.publicurl, b.adminurl)
            expect = ('2',
                      'http://example.com:1234/go',
                      'http://example.com:9876/adm')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))

            # New-style options
            # Test create args
            shell('endpoint-create '
                  '--service-id=3 --publicurl=http://example.com:4321/go '
                  '--adminurl=http://example.com:9876/adm')
            assert do_shell_mock.called
            ((a, b), c) = do_shell_mock.call_args
            actual = (b.os_auth_url, b.os_password, b.os_tenant_id,
                      b.os_tenant_name, b.os_username,
                      b.os_identity_api_version)
            expect = (DEFAULT_AUTH_URL, DEFAULT_PASSWORD, DEFAULT_TENANT_ID,
                      DEFAULT_TENANT_NAME, DEFAULT_USERNAME, '')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))
            actual = (b.service, b.publicurl, b.adminurl)
            expect = ('3',
                      'http://example.com:4321/go',
                      'http://example.com:9876/adm')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))

            # New-style options
            # Test create args
            shell('endpoint-create '
                  '--service=3 --publicurl=http://example.com:4321/go '
                  '--adminurl=http://example.com:9876/adm')
            assert do_shell_mock.called
            ((a, b), c) = do_shell_mock.call_args
            actual = (b.os_auth_url, b.os_password, b.os_tenant_id,
                      b.os_tenant_name, b.os_username,
                      b.os_identity_api_version)
            expect = (DEFAULT_AUTH_URL, DEFAULT_PASSWORD, DEFAULT_TENANT_ID,
                      DEFAULT_TENANT_NAME, DEFAULT_USERNAME, '')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))
            actual = (b.service, b.publicurl, b.adminurl)
            expect = ('3',
                      'http://example.com:4321/go',
                      'http://example.com:9876/adm')
            self.assertTrue(all([x == y for x, y in zip(actual, expect)]))

    def test_shell_keyboard_interrupt(self):
        shell_mock = mock.MagicMock()
        with mock.patch('keystoneclient.shell.OpenStackIdentityShell.main',
                        shell_mock):
            try:
                shell_mock.side_effect = KeyboardInterrupt()
                openstack_shell.main()
            except SystemExit as ex:
                self.assertEqual(130, ex.code)



# Copyright 2010 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import six

from keystoneclient.generic import client
from keystoneclient.i18n import _
from keystoneclient import utils


CLIENT_CLASS = client.Client


@utils.unauthenticated
def do_discover(cs, args):
    """Discover Keystone servers, supported API versions and extensions."""
    if cs.endpoint:
        versions = cs.discover(cs.endpoint)
    elif cs.auth_url:
        versions = cs.discover(cs.auth_url)
    else:
        versions = cs.discover()
    if versions:
        if 'message' in versions:
            print(versions['message'])
        for key, version in six.iteritems(versions):
            if key != 'message':
                print(_("    - supports version %(id)s (%(status)s) here "
                        "%(url)s") %
                      version)
                extensions = cs.discover_extensions(version['url'])
                if extensions:
                    for key, extension in six.iteritems(extensions):
                        if key != 'message':
                            print(_("        - and %(key)s: %(extension)s") %
                                  {'key': key, 'extension': extension})
    else:
        print(_("No Keystone-compatible endpoint found"))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import sys

import mock
from oslo_serialization import jsonutils
import six
from testtools import matchers

from keystoneclient import fixture
from keystoneclient.tests.unit.v2_0 import utils


DEFAULT_USERNAME = 'username'
DEFAULT_PASSWORD = 'password'
DEFAULT_TENANT_ID = 'tenant_id'
DEFAULT_TENANT_NAME = 'tenant_name'
DEFAULT_AUTH_URL = 'http://127.0.0.1:5000/v2.0/'
DEFAULT_ADMIN_URL = 'http://127.0.0.1:35357/v2.0/'


class ShellTests(utils.TestCase):

    TEST_URL = DEFAULT_ADMIN_URL

    def setUp(self):
        """Patch os.environ to avoid required auth info."""

        super(ShellTests, self).setUp()

        self.addCleanup(setattr, os, 'environ', os.environ.copy())
        os.environ = {
            'OS_USERNAME': DEFAULT_USERNAME,
            'OS_PASSWORD': DEFAULT_PASSWORD,
            'OS_TENANT_ID': DEFAULT_TENANT_ID,
            'OS_TENANT_NAME': DEFAULT_TENANT_NAME,
            'OS_AUTH_URL': DEFAULT_AUTH_URL,
        }
        import keystoneclient.shell
        self.shell = keystoneclient.shell.OpenStackIdentityShell()

        self.token = fixture.V2Token()
        self.token.set_scope()
        svc = self.token.add_service('identity')
        svc.add_endpoint(public=DEFAULT_AUTH_URL,
                         admin=DEFAULT_ADMIN_URL)

        self.stub_auth(json=self.token, base_url=DEFAULT_AUTH_URL)

    def run_command(self, cmd):
        orig = sys.stdout
        try:
            sys.stdout = six.StringIO()
            if isinstance(cmd, list):
                self.shell.main(cmd)
            else:
                self.shell.main(cmd.split())
        except SystemExit:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            self.assertEqual(exc_value.code, 0)
        finally:
            out = sys.stdout.getvalue()
            sys.stdout.close()
            sys.stdout = orig
        return out

    def assert_called(self, method, path, base_url=TEST_URL):
        self.assertEqual(method, self.requests_mock.last_request.method)
        self.assertEqual(base_url + path.lstrip('/'),
                         self.requests_mock.last_request.url)

    def test_user_list(self):
        self.stub_url('GET', ['users'], json={'users': []})
        self.run_command('user-list')
        self.assert_called('GET', '/users')

    def test_user_create(self):
        self.stub_url('POST', ['users'], json={'user': {}})
        self.run_command('user-create --name new-user')

        self.assert_called('POST', '/users')
        self.assertRequestBodyIs(json={'user': {'email': None,
                                                'password': None,
                                                'enabled': True,
                                                'name': 'new-user',
                                                'tenantId': None}})

    @mock.patch('sys.stdin', autospec=True)
    def test_user_create_password_prompt(self, mock_stdin):
        self.stub_url('POST', ['users'], json={'user': {}})

        with mock.patch('getpass.getpass') as mock_getpass:
            del(os.environ['OS_PASSWORD'])
            mock_stdin.isatty = lambda: True
            mock_getpass.return_value = 'newpass'
            self.run_command('user-create --name new-user --pass')

            self.assert_called('POST', '/users')
            self.assertRequestBodyIs(json={'user': {'email': None,
                                                    'password': 'newpass',
                                                    'enabled': True,
                                                    'name': 'new-user',
                                                    'tenantId': None}})

    def test_user_get(self):
        self.stub_url('GET', ['users', '1'],
                      json={'user': {'id': '1'}})
        self.run_command('user-get 1')
        self.assert_called('GET', '/users/1')

    def test_user_delete(self):
        self.stub_url('GET', ['users', '1'],
                      json={'user': {'id': '1'}})
        self.stub_url('DELETE', ['users', '1'])
        self.run_command('user-delete 1')
        self.assert_called('DELETE', '/users/1')

    def test_user_password_update(self):
        self.stub_url('GET', ['users', '1'],
                      json={'user': {'id': '1'}})
        self.stub_url('PUT', ['users', '1', 'OS-KSADM', 'password'])
        self.run_command('user-password-update --pass newpass 1')
        self.assert_called('PUT', '/users/1/OS-KSADM/password')

    def test_user_update(self):
        self.stub_url('PUT', ['users', '1'])
        self.stub_url('GET', ['users', '1'],
                      json={"user": {"tenantId": "1",
                                     "enabled": "true",
                                     "id": "1",
                                     "name": "username"}})

        self.run_command('user-update --name new-user1'
                         ' --email user@email.com --enabled true 1')
        self.assert_called('PUT', '/users/1')
        body = {'user': {'id': '1', 'email': 'user@email.com',
                         'enabled': True, 'name': 'new-user1'}}
        self.assertRequestBodyIs(json=body)

        required = 'User not updated, no arguments present.'
        out = self.run_command('user-update 1')
        self.assertThat(out, matchers.MatchesRegex(required))

        self.run_command(['user-update', '--email', '', '1'])
        self.assert_called('PUT', '/users/1')
        self.assertRequestBodyIs(json={'user': {'id': '1', 'email': ''}})

    def test_role_create(self):
        self.stub_url('POST', ['OS-KSADM', 'roles'], json={'role': {}})
        self.run_command('role-create --name new-role')
        self.assert_called('POST', '/OS-KSADM/roles')
        self.assertRequestBodyIs(json={"role": {"name": "new-role"}})

    def test_role_get(self):
        self.stub_url('GET', ['OS-KSADM', 'roles', '1'],
                      json={'role': {'id': '1'}})
        self.run_command('role-get 1')
        self.assert_called('GET', '/OS-KSADM/roles/1')

    def test_role_list(self):
        self.stub_url('GET', ['OS-KSADM', 'roles'], json={'roles': []})
        self.run_command('role-list')
        self.assert_called('GET', '/OS-KSADM/roles')

    def test_role_delete(self):
        self.stub_url('GET', ['OS-KSADM', 'roles', '1'],
                      json={'role': {'id': '1'}})
        self.stub_url('DELETE', ['OS-KSADM', 'roles', '1'])
        self.run_command('role-delete 1')
        self.assert_called('DELETE', '/OS-KSADM/roles/1')

    def test_user_role_add(self):
        self.stub_url('GET', ['users', '1'],
                      json={'user': {'id': '1'}})
        self.stub_url('GET', ['OS-KSADM', 'roles', '1'],
                      json={'role': {'id': '1'}})

        self.stub_url('PUT', ['users', '1', 'roles', 'OS-KSADM', '1'])
        self.run_command('user-role-add --user_id 1 --role_id 1')
        self.assert_called('PUT', '/users/1/roles/OS-KSADM/1')

    def test_user_role_list(self):
        self.stub_url('GET', ['tenants', self.token.tenant_id],
                      json={'tenant': {'id': self.token.tenant_id}})
        self.stub_url('GET', ['tenants', self.token.tenant_id,
                              'users', self.token.user_id, 'roles'],
                      json={'roles': []})

        url = '/tenants/%s/users/%s/roles' % (self.token.tenant_id,
                                              self.token.user_id)

        self.run_command('user-role-list --user_id %s --tenant-id %s' %
                         (self.token.user_id, self.token.tenant_id))
        self.assert_called('GET', url)

        self.run_command('user-role-list --user_id %s' % self.token.user_id)
        self.assert_called('GET', url)

        self.run_command('user-role-list')
        self.assert_called('GET', url)

    def test_user_role_remove(self):
        self.stub_url('GET', ['users', '1'],
                      json={'user': {'id': 1}})
        self.stub_url('GET', ['OS-KSADM', 'roles', '1'],
                      json={'role': {'id': 1}})
        self.stub_url('DELETE',
                      ['users', '1', 'roles', 'OS-KSADM', '1'])

        self.run_command('user-role-remove --user_id 1 --role_id 1')
        self.assert_called('DELETE', '/users/1/roles/OS-KSADM/1')

    def test_tenant_create(self):
        self.stub_url('POST', ['tenants'], json={'tenant': {}})
        self.run_command('tenant-create --name new-tenant')
        self.assertRequestBodyIs(json={"tenant": {"enabled": True,
                                                  "name": "new-tenant",
                                                  "description": None}})

    def test_tenant_get(self):
        self.stub_url('GET', ['tenants', '2'], json={'tenant': {}})
        self.run_command('tenant-get 2')
        self.assert_called('GET', '/tenants/2')

    def test_tenant_list(self):
        self.stub_url('GET', ['tenants'], json={'tenants': []})
        self.run_command('tenant-list')
        self.assert_called('GET', '/tenants')

    def test_tenant_update(self):
        self.stub_url('GET', ['tenants', '1'],
                      json={'tenant': {'id': '1'}})
        self.stub_url('GET', ['tenants', '2'],
                      json={'tenant': {'id': '2'}})
        self.stub_url('POST', ['tenants', '2'],
                      json={'tenant': {'id': '2'}})
        self.run_command('tenant-update'
                         ' --name new-tenant1 --enabled false'
                         ' --description desc 2')
        self.assert_called('POST', '/tenants/2')
        self.assertRequestBodyIs(json={"tenant": {"enabled": False,
                                                  "id": "2",
                                                  "description": "desc",
                                                  "name": "new-tenant1"}})

        required = 'Tenant not updated, no arguments present.'
        out = self.run_command('tenant-update 1')
        self.assertThat(out, matchers.MatchesRegex(required))

    def test_tenant_delete(self):
        self.stub_url('GET', ['tenants', '2'],
                      json={'tenant': {'id': '2'}})
        self.stub_url('DELETE', ['tenants', '2'])
        self.run_command('tenant-delete 2')
        self.assert_called('DELETE', '/tenants/2')

    def test_service_create_with_required_arguments_only(self):
        self.stub_url('POST', ['OS-KSADM', 'services'],
                      json={'OS-KSADM:service': {}})
        self.run_command('service-create --type compute')
        self.assert_called('POST', '/OS-KSADM/services')
        json = {"OS-KSADM:service": {"type": "compute",
                                     "name": None,
                                     "description": None}}
        self.assertRequestBodyIs(json=json)

    def test_service_create_with_all_arguments(self):
        self.stub_url('POST', ['OS-KSADM', 'services'],
                      json={'OS-KSADM:service': {}})
        self.run_command('service-create --type compute '
                         '--name service1 --description desc1')
        self.assert_called('POST', '/OS-KSADM/services')
        json = {"OS-KSADM:service": {"type": "compute",
                                     "name": "service1",
                                     "description": "desc1"}}
        self.assertRequestBodyIs(json=json)

    def test_service_get(self):
        self.stub_url('GET', ['OS-KSADM', 'services', '1'],
                      json={'OS-KSADM:service': {'id': '1'}})
        self.run_command('service-get 1')
        self.assert_called('GET', '/OS-KSADM/services/1')

    def test_service_list(self):
        self.stub_url('GET', ['OS-KSADM', 'services'],
                      json={'OS-KSADM:services': []})
        self.run_command('service-list')
        self.assert_called('GET', '/OS-KSADM/services')

    def test_service_delete(self):
        self.stub_url('GET', ['OS-KSADM', 'services', '1'],
                      json={'OS-KSADM:service': {'id': 1}})
        self.stub_url('DELETE', ['OS-KSADM', 'services', '1'])
        self.run_command('service-delete 1')
        self.assert_called('DELETE', '/OS-KSADM/services/1')

    def test_catalog(self):
        self.run_command('catalog')
        self.run_command('catalog --service compute')

    def test_ec2_credentials_create(self):
        self.stub_url('POST',
                      ['users', self.token.user_id, 'credentials', 'OS-EC2'],
                      json={'credential': {}})

        url = '/users/%s/credentials/OS-EC2' % self.token.user_id
        self.run_command('ec2-credentials-create --tenant-id 1 '
                         '--user-id %s' % self.token.user_id)
        self.assert_called('POST', url)
        self.assertRequestBodyIs(json={'tenant_id': '1'})

        self.run_command('ec2-credentials-create --tenant-id 1')
        self.assert_called('POST', url)
        self.assertRequestBodyIs(json={'tenant_id': '1'})

        self.run_command('ec2-credentials-create')
        self.assert_called('POST', url)
        self.assertRequestBodyIs(json={'tenant_id': self.token.tenant_id})

    def test_ec2_credentials_delete(self):
        self.stub_url('DELETE',
                      ['users', self.token.user_id,
                       'credentials', 'OS-EC2', '2'])
        self.run_command('ec2-credentials-delete --access 2 --user-id %s' %
                         self.token.user_id)

        url = '/users/%s/credentials/OS-EC2/2' % self.token.user_id
        self.assert_called('DELETE', url)

        self.run_command('ec2-credentials-delete --access 2')
        self.assert_called('DELETE', url)

    def test_ec2_credentials_list(self):
        self.stub_url('GET',
                      ['users', self.token.user_id, 'credentials', 'OS-EC2'],
                      json={'credentials': []})
        self.run_command('ec2-credentials-list --user-id %s'
                         % self.token.user_id)

        url = '/users/%s/credentials/OS-EC2' % self.token.user_id
        self.assert_called('GET', url)

        self.run_command('ec2-credentials-list')
        self.assert_called('GET', url)

    def test_ec2_credentials_get(self):
        self.stub_url('GET',
                      ['users', '1', 'credentials', 'OS-EC2', '2'],
                      json={'credential': {}})
        self.run_command('ec2-credentials-get --access 2 --user-id 1')
        self.assert_called('GET', '/users/1/credentials/OS-EC2/2')

    def test_bootstrap(self):
        user = {'user': {'id': '1'}}
        role = {'role': {'id': '1'}}
        tenant = {'tenant': {'id': '1'}}

        token = fixture.V2Token(user_id=1, tenant_id=1)
        token.add_role(id=1)
        svc = token.add_service('identity')
        svc.add_endpoint(public=DEFAULT_AUTH_URL,
                         admin=DEFAULT_ADMIN_URL)

        self.stub_auth(json=token)

        self.stub_url('POST', ['OS-KSADM', 'roles'], json=role)
        self.stub_url('GET', ['OS-KSADM', 'roles', '1'], json=role)
        self.stub_url('POST', ['tenants'], json=tenant)
        self.stub_url('GET', ['tenants', '1'], json=tenant)
        self.stub_url('POST', ['users'], json=user)
        self.stub_url('GET', ['users', '1'], json=user)
        self.stub_url('PUT',
                      ['tenants', '1', 'users', '1', 'roles', 'OS-KSADM', '1'],
                      json=role)

        self.run_command('bootstrap --user-name new-user'
                         ' --pass 1 --role-name admin'
                         ' --tenant-name new-tenant')

        def called_anytime(method, path, json=None):
            test_url = self.TEST_URL.strip('/')
            for r in self.requests_mock.request_history:
                if not r.method == method:
                    continue
                if not r.url == test_url + path:
                    continue

                if json:
                    json_body = jsonutils.loads(r.body)
                    if not json_body == json:
                        continue

                return True

            raise AssertionError('URL never called')

        called_anytime('POST', '/users', {'user': {'email': None,
                                                   'password': '1',
                                                   'enabled': True,
                                                   'name': 'new-user',
                                                   'tenantId': None}})

        called_anytime('POST', '/tenants', {"tenant": {"enabled": True,
                                                       "name": "new-tenant",
                                                       "description": None}})

        called_anytime('POST', '/OS-KSADM/roles',
                       {"role": {"name": "admin"}})

        called_anytime('PUT', '/tenants/1/users/1/roles/OS-KSADM/1')

    def test_bash_completion(self):
        self.run_command('bash-completion')

    def test_help(self):
        out = self.run_command('help')
        required = 'usage: keystone'
        self.assertThat(out, matchers.MatchesRegex(required))

    def test_password_update(self):
        self.stub_url('PATCH',
                      ['OS-KSCRUD', 'users', self.token.user_id],
                      base_url=DEFAULT_AUTH_URL)
        self.run_command('password-update --current-password oldpass'
                         ' --new-password newpass')
        self.assert_called('PATCH',
                           '/OS-KSCRUD/users/%s' % self.token.user_id,
                           base_url=DEFAULT_AUTH_URL)
        self.assertRequestBodyIs(json={'user': {'original_password': 'oldpass',
                                                'password': 'newpass'}})

    def test_endpoint_create(self):
        self.stub_url('GET', ['OS-KSADM', 'services', '1'],
                      json={'OS-KSADM:service': {'id': '1'}})
        self.stub_url('POST', ['endpoints'], json={'endpoint': {}})
        self.run_command('endpoint-create --service-id 1 '
                         '--publicurl=http://example.com:1234/go')
        self.assert_called('POST', '/endpoints')
        json = {'endpoint': {'adminurl': None,
                             'service_id': '1',
                             'region': 'regionOne',
                             'internalurl': None,
                             'publicurl': "http://example.com:1234/go"}}
        self.assertRequestBodyIs(json=json)

    def test_endpoint_list(self):
        self.stub_url('GET', ['endpoints'], json={'endpoints': []})
        self.run_command('endpoint-list')
        self.assert_called('GET', '/endpoints')




# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import re

from tempest_lib.cli import base
from tempest_lib import exceptions


class SimpleReadOnlyKeystoneClientTest(base.ClientTestBase):
    """Basic, read-only tests for Keystone CLI client.

    Checks return values and output of read-only commands.
    These tests do not presume any content, nor do they create
    their own. They only verify the structure of output if present.
    """

    def _get_clients(self):
        path = os.path.join(os.path.abspath('.'), '.tox/functional/bin')
        cli_dir = os.environ.get('OS_KEYSTONECLIENT_EXEC_DIR', path)

        return base.CLIClient(
            username=os.environ.get('OS_USERNAME'),
            password=os.environ.get('OS_PASSWORD'),
            tenant_name=os.environ.get('OS_TENANT_NAME'),
            uri=os.environ.get('OS_AUTH_URL'),
            cli_dir=cli_dir)

    def keystone(self, *args, **kwargs):
        return self.clients.keystone(*args, **kwargs)

    def test_admin_fake_action(self):
        self.assertRaises(exceptions.CommandFailed,
                          self.keystone,
                          'this-does-not-exist')

    def test_admin_catalog_list(self):
        out = self.keystone('catalog')
        catalog = self.parser.details_multiple(out, with_label=True)
        for svc in catalog:
            if svc.get('__label'):
                self.assertTrue(svc['__label'].startswith('Service:'),
                                msg=('Invalid beginning of service block: '
                                     '%s' % svc['__label']))
            # check that region and publicURL exists. One might also
            # check for adminURL and internalURL. id seems to be optional
            # and is missing in the catalog backend
            self.assertIn('publicURL', svc)
            self.assertIn('region', svc)

    def test_admin_endpoint_list(self):
        out = self.keystone('endpoint-list')
        endpoints = self.parser.listing(out)
        self.assertTableStruct(endpoints, [
            'id', 'region', 'publicurl', 'internalurl',
            'adminurl', 'service_id'])

    def test_admin_endpoint_service_match(self):
        endpoints = self.parser.listing(self.keystone('endpoint-list'))
        services = self.parser.listing(self.keystone('service-list'))
        svc_by_id = {}
        for svc in services:
            svc_by_id[svc['id']] = svc
        for endpoint in endpoints:
            self.assertIn(endpoint['service_id'], svc_by_id)

    def test_admin_role_list(self):
        roles = self.parser.listing(self.keystone('role-list'))
        self.assertTableStruct(roles, ['id', 'name'])

    def test_admin_service_list(self):
        services = self.parser.listing(self.keystone('service-list'))
        self.assertTableStruct(services, ['id', 'name', 'type', 'description'])

    def test_admin_tenant_list(self):
        tenants = self.parser.listing(self.keystone('tenant-list'))
        self.assertTableStruct(tenants, ['id', 'name', 'enabled'])

    def test_admin_user_list(self):
        users = self.parser.listing(self.keystone('user-list'))
        self.assertTableStruct(users, [
            'id', 'name', 'enabled', 'email'])

    def test_admin_user_role_list(self):
        user_roles = self.parser.listing(self.keystone('user-role-list'))
        self.assertTableStruct(user_roles, [
            'id', 'name', 'user_id', 'tenant_id'])

    def test_admin_discover(self):
        discovered = self.keystone('discover')
        self.assertIn('Keystone found at http', discovered)
        self.assertIn('supports version', discovered)

    def test_admin_help(self):
        help_text = self.keystone('help')
        lines = help_text.split('\n')
        self.assertFirstLineStartsWith(lines, 'usage: keystone')

        commands = []
        cmds_start = lines.index('Positional arguments:')
        cmds_end = lines.index('Optional arguments:')
        command_pattern = re.compile('^ {4}([a-z0-9\-\_]+)')
        for line in lines[cmds_start:cmds_end]:
            match = command_pattern.match(line)
            if match:
                commands.append(match.group(1))
        commands = set(commands)
        wanted_commands = set(('catalog', 'endpoint-list', 'help',
                               'token-get', 'discover', 'bootstrap'))
        self.assertFalse(wanted_commands - commands)

    def test_admin_bashcompletion(self):
        self.keystone('bash-completion')

    def test_admin_ec2_credentials_list(self):
        creds = self.keystone('ec2-credentials-list')
        creds = self.parser.listing(creds)
        self.assertTableStruct(creds, ['tenant', 'access', 'secret'])

    # Optional arguments:

    def test_admin_version(self):
        self.keystone('', flags='--version')

    def test_admin_debug_list(self):
        self.keystone('catalog', flags='--debug')

    def test_admin_timeout(self):
        self.keystone('catalog', flags='--timeout %d' % 15)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from positional import positional

from keystoneclient import access
from keystoneclient import base


def _calc_id(token):
    if isinstance(token, access.AccessInfo):
        return token.auth_token

    return base.getid(token)


class TokenManager(object):
    """Manager class for manipulating Identity tokens."""

    def __init__(self, client):
        self._client = client

    def revoke_token(self, token):
        """Revoke a token.

        :param token: Token to be revoked. This can be an instance of
                      :py:class:`keystoneclient.access.AccessInfo` or a string
                      token_id.
        """

        token_id = _calc_id(token)
        headers = {'X-Subject-Token': token_id}
        return self._client.delete('/auth/tokens', headers=headers)

    def get_revoked(self):
        """Get revoked tokens list.

        :returns: A dict containing "signed" which is a CMS formatted string.
        :rtype: dict

        """

        resp, body = self._client.get('/auth/tokens/OS-PKI/revoked')
        return body

    @positional.method(1)
    def get_token_data(self, token, include_catalog=True):
        """Fetch the data about a token from the identity server.

        :param str token: The token id.
        :param bool include_catalog: If False, the response is requested to not
                                     include the catalog.

        :rtype: dict
        """
        headers = {'X-Subject-Token': token}

        url = '/auth/tokens'
        if not include_catalog:
            url += '?nocatalog'

        resp, body = self._client.get(url, headers=headers)
        return body

    @positional.method(1)
    def validate(self, token, include_catalog=True):
        """Validate a token.

        :param token: Token to be validated. This can be an instance of
                      :py:class:`keystoneclient.access.AccessInfo` or a string
                      token_id.
        :param include_catalog: If False, the response is requested to not
                                include the catalog.

        :rtype: :py:class:`keystoneclient.access.AccessInfoV3`

        """

        token_id = _calc_id(token)
        body = self.get_token_data(token_id, include_catalog=include_catalog)
        return access.AccessInfo.factory(auth_token=token_id, body=body)
